import re
from functools import wraps
import asyncio
from .server import run_server
from .request import Request
from .response import build_response
from .middleware import MiddlewareManager
from .static import serve_static

class Zero:
    def __init__(self, static_folder="static"):
        self.routes = []
        self.error_handler = None
        self.middleware = MiddlewareManager()
        self.static_folder = static_folder

    def route(self, path, methods=["GET"]):
        def decorator(handler):
            pattern = self._convert_path_to_regex(path)
            self.routes.append({
                "pattern": pattern,
                "methods": methods,
                "handler": handler
            })
            return handler
        return decorator

    def _convert_path_to_regex(self, path):
        regex = re.sub(r'<(\w+)>', r'(?P<\1>[^/]+)', path)
        return re.compile(f"^{regex}$")

    def use(self, middleware_func):
        self.middleware.add(middleware_func)

    def run(self, host="127.0.0.1", port=8000):
        run_server(self, host, port)

    async def handle_request(self, request):
        for middleware in self.middleware.before_request:
            request = middleware(request) or request

        if request.path.startswith("/static/"):
            return await serve_static(request)

        for route in self.routes:
            match = route["pattern"].match(request.path)
            if match and request.method in route["methods"]:
                kwargs = match.groupdict()
                handler = route["handler"]

                try:
                    if asyncio.iscoroutinefunction(handler):
                        response = await handler(request, **kwargs)
                    else:
                        response = handler(request, **kwargs)
                    return response
                except Exception as e:
                    if self.error_handler:
                        return self.error_handler(e)
                    else:
                        return {"error": str(e)}, 500

        return "Not Found", 404

    def errorhandler(self, handler):
        self.error_handler = handler



import json

def build_response(data):
    if isinstance(data, tuple):
        body, status_code = data
    else:
        body, status_code = data, 200

    if isinstance(body, dict):
        body_str = json.dumps(body)
        headers = "Content-Type: application/json\r\n"
    else:
        body_str = str(body)
        headers = "Content-Type: text/plain\r\n"

    status_line = f"HTTP/1.1 {status_code} OK\r\n"
    body_len = f"Content-Length: {len(body_str)}\r\n"
    response = status_line + headers + body_len + "\r\n" + body_str
    return response.encode()



import os
import asyncio

async def serve_static(request):
    filename = request.path[len("/static/"):]
    filepath = os.path.join("static", filename)

    if not os.path.exists(filepath):
        return "File Not Found", 404

    _, ext = os.path.splitext(filename)
    content_types = {
        ".html": "text/html",
        ".css": "text/css",
        ".js": "application/javascript",
        ".png": "image/png",
        ".jpg": "image/jpeg",
    }

    content_type = content_types.get(ext, "application/octet-stream")

    with open(filepath, "rb") as f:
        data = f.read()

    return (f"HTTP/1.1 200 OK\r\nContent-Type: {content_type}\r\n"
            f"Content-Length: {len(data)}\r\n\r\n").encode() + data



import socket
import threading
import asyncio
from .request import parse_request
from .response import build_response

def run_server(app, host="127.0.0.1", port=8000):
    def _handle_client(conn, addr, app):
        data = conn.recv(4096)
        if not data:
            return

        request = parse_request(data)
        if not request:
            conn.sendall(b"HTTP/1.1 400 Bad Request\r\n\r\n")
            return

        loop = asyncio.new_event_loop()
        response_data = loop.run_until_complete(app.handle_request(request))
        conn.sendall(build_response(response_data))
        conn.close()

    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind((host, port))
        sock.listen(100)
        print(f"Server running on http://{host}:{port}")

        while True:
            conn, addr = sock.accept()
            thread = threading.Thread(target=_handle_client, args=(conn, addr, app))
            thread.start()



# myzero/request.py

import json
from urllib.parse import parse_qs

class Request:
    """
    A simple HTTP request object that wraps parsed data.
    """
    def __init__(self, method, path, headers, body, data=None):
        self.method = method       # HTTP method (GET, POST)
        self.path = path           # URL path
        self.headers = headers     # Parsed headers
        self.body = body           # Raw body bytes
        self.data = data           # Parsed JSON or form data


def parse_request(raw_data):
    """
    Parses raw HTTP request bytes into a Request object.
    """
    try:
        # Decode raw data safely
        raw_text = raw_data.decode('utf-8', errors='ignore')
        headers_end = raw_text.find("\r\n\r\n")

        if headers_end == -1:
            print("No headers/body separator found")
            return None

        header_part = raw_text[:headers_end]
        lines = header_part.split("\r\n")

        if len(lines) < 1:
            print("Malformed request line")
            return None

        # Parse method, path, HTTP version
        method, path, _ = lines[0].split(" ", 2)

        # Parse headers
        headers = {}
        for line in lines[1:]:
            if ": " in line:
                key, value = line.split(": ", 1)
                headers[key.strip()] = value.strip()

        # Extract body
        body_start = headers_end + 4
        body = raw_data[body_start:] if len(raw_data) > body_start else b""

        content_type = headers.get("Content-Type", "").lower()
        data = None

        # Handle JSON
        if content_type == "application/json":
            try:
                data = json.loads(body.decode('utf-8'))
            except (json.JSONDecodeError, UnicodeDecodeError) as e:
                print(f"JSON decode error: {e}")
                data = None

        # Handle URL-encoded form
        elif content_type.startswith("application/x-www-form-urlencoded"):
            try:
                data = parse_qs(body.decode('utf-8'))
            except Exception as e:
                print(f"Form decode error: {e}")

        # Return Request object
        return Request(method, path, headers, body, data)

    except Exception as e:
        print(f"Request parsing error: {e}")
        return None


# myzero/__init__.py

__version__ = "0.1.3"

from .app import Zero
from .request import Request
from .response import build_response
from .server import run_server
from .middleware import MiddlewareManager


# setup.py

from setuptools import setup, find_packages

setup(
    name="zerofl",
    version="0.1.5",
    packages=find_packages(),  # This will include 'myzero'
    description="A lightweight, production-ready micro web framework in Python.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="MD Imran",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)